from django.conf import settings


class RatingValues:
    DEFAULT_USER_TYPES = (
        ('U', 'USER_TYPE'),
    )
    DEFAULT_ELEMENT_TYPES = (
        ('E', 'ELEMENT_TYPE'),
    )

    VALID_USER_TYPES = getattr(settings, 'RATING_VALID_USER_TYPES', DEFAULT_USER_TYPES)
    VALID_ELEMENT_TYPES = getattr(settings, 'RATING_VALID_ELEMENT_TYPES', DEFAULT_ELEMENT_TYPES)
    MIN_SCORE = getattr(settings, 'RATING_MIN_SCORE', 0)
    MAX_SCORE = getattr(settings, 'RATING_MAX_SCORE', 10)



# Generated by Django 2.2.3 on 2019-08-08 11:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('rating', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='objectrating',
            options={'ordering': ('id',), 'verbose_name': 'Rating'},
        ),
    ]



from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.contrib.auth.models import User
from django_extensions.db.models import TimeStampedModel


class RatingElement(TimeStampedModel):
    element_type = models.CharField(max_length=50)
    score = models.SmallIntegerField(blank=True, null=True)
    comment = models.CharField(blank=True, null=True, max_length=512)

    class Meta:
        ordering = ('id', )

    def __str__(self):
        return f'{self.id} - {self.element_type} - {self.score} - {self.comment}'


class ObjectRating(TimeStampedModel):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    user_type = models.CharField(blank=True, null=True, max_length=50)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey()

    elements = models.ManyToManyField(RatingElement)

    class Meta:
        verbose_name = 'Rating'
        ordering = ('id', )



# Generated by Django 2.1.1 on 2019-04-15 13:20

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='ObjectRating',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('user_type', models.CharField(blank=True, max_length=50, null=True)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'ordering': ('id',),
            },
        ),
        migrations.CreateModel(
            name='RatingElement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('element_type', models.CharField(max_length=50)),
                ('score', models.SmallIntegerField(blank=True, null=True)),
                ('comment', models.CharField(blank=True, max_length=512, null=True)),
            ],
            options={
                'ordering': ('id',),
            },
        ),
        migrations.AddField(
            model_name='objectrating',
            name='elements',
            field=models.ManyToManyField(to='rating.RatingElement'),
        ),
        migrations.AddField(
            model_name='objectrating',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]



from rest_framework.routers import DefaultRouter

from .views import ObjectRatingViewSet, RatingElementViewSet

router = DefaultRouter()
router.register(r'objectratings', ObjectRatingViewSet)
router.register(r'ratingelements', RatingElementViewSet)

urlpatterns = router.urls



from rest_framework.permissions import IsAuthenticated
from rest_framework.viewsets import ModelViewSet

from .models import ObjectRating, RatingElement
from .serializers import ObjectRatingSerializer, RatingElementSerializer


class RatingElementViewSet(ModelViewSet):
    queryset = RatingElement.objects.all()
    serializer_class = RatingElementSerializer
    permission_classes = (IsAuthenticated, )


class ObjectRatingViewSet(ModelViewSet):
    queryset = ObjectRating.objects.all()
    serializer_class = ObjectRatingSerializer
    permission_classes = (IsAuthenticated, )



from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType
from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import APITestCase, APIClient

from .models import RatingElement, ObjectRating
from .rating_values import RatingValues


class TestObjectRatingAPI(APITestCase):
    PASS = 'secretpass'

    def setUp(self):
        super().setUp()

        self.user_1 = User.objects.create_user(
            'user_1@test.com', email='user_1@test.com', password=self.PASS)
        self.user_2 = User.objects.create_user(
            'user_2@test.com', email='user_2@test.com', password=self.PASS)

        self.user_client = APIClient()
        self.user_client.login(username='user_1@test.com', password=self.PASS)

        self.perm = Permission.objects.first()
        self.perm_c_t = ContentType.objects.get(model='permission')

        self.element_type = RatingValues.VALID_ELEMENT_TYPES[0][0]
        self.user_type = RatingValues.VALID_USER_TYPES[0][0]

    def test_create_rating_element_success(self):
        data = {
            'element_type': self.element_type,
            'score': 5,
        }
        url = reverse('ratingelement-list')
        response = self.user_client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        resp_json = response.json()
        self.assertEqual(resp_json['element_type'], self.element_type)
        self.assertEqual(resp_json['score'], 5)
        self.assertEqual(resp_json['comment'], None)

    def test_create_rating_element_with_invalid_type_fail(self):
        data = {
            'element_type': 'invalid element type',
            'score': 5,
        }
        url = reverse('ratingelement-list')
        response = self.user_client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.json(), {'element_type': ['Invalid element type.']})

    def test_create_rating_element_with_score_too_low_fail(self):
        data = {
            'element_type': self.element_type,
            'score': -1,
        }
        url = reverse('ratingelement-list')
        response = self.user_client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.json(), {'score': ['Invalid score.']})

    def test_create_rating_element_with_score_too_high_fail(self):
        data = {
            'element_type': self.element_type,
            'score': 11,
        }
        url = reverse('ratingelement-list')
        response = self.user_client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.json(), {'score': ['Invalid score.']})

    def test_get_rating_element(self):
        element = RatingElement.objects.create(
            element_type=self.element_type, score=10, comment='super'
        )
        url = reverse('ratingelement-detail', args=(element.id,))
        response = self.user_client.get(url, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        resp_json = response.json()
        self.assertEqual(resp_json['element_type'], self.element_type)
        self.assertEqual(resp_json['score'], 10)
        self.assertEqual(resp_json['comment'], 'super')

    def test_list_rating_elements(self):
        RatingElement.objects.create(element_type=self.element_type, score=10, comment='super')
        RatingElement.objects.create(element_type=self.element_type, score=9, comment='good')
        url = reverse('ratingelement-list')
        response = self.user_client.get(url, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.json()), 2)

    def test_update_rating_element(self):
        element = RatingElement.objects.create(element_type=self.element_type, score=5,)
        data = {'score': 4}
        url = reverse('ratingelement-detail', args=(element.id,))
        response = self.user_client.patch(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        resp_json = response.json()
        self.assertEqual(resp_json['element_type'], self.element_type)
        self.assertEqual(resp_json['score'], 4)

    def test_delete_rating_element(self):
        element = RatingElement.objects.create(element_type=self.element_type, score=5,)
        url = reverse('ratingelement-detail', args=(element.id,))
        response = self.user_client.delete(url, format='json')
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    def test_create_object_rating_with_invalid_user_type_fail(self):
        element = RatingElement.objects.create(element_type=self.element_type, score=10, comment='super')
        data = {
            'user_type': 'invalid type',
            'object_id': self.perm.id,
            'content_type': self.perm_c_t.id,
            'elements': [element.id]
        }
        url = reverse('objectrating-list')
        response = self.user_client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.json(), {'user_type': ['Invalid user type.']})

    def test_create_object_rating_success(self):
        element = RatingElement.objects.create(element_type=self.element_type, score=10, comment='super')
        data = {
            'user_type': self.user_type,
            'object_id': self.perm.id,
            'content_type': self.perm_c_t.id,
            'elements': [element.id]
        }
        url = reverse('objectrating-list')
        response = self.user_client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        resp_json = response.json()
        self.assertEqual(resp_json['user_type'], self.user_type)
        self.assertEqual(resp_json['object_id'], self.perm.id)
        self.assertEqual(resp_json['content_type'], self.perm_c_t.id)
        self.assertEqual(resp_json['elements'], [element.id])

    def test_list_objects_ratings(self):
        element_1 = RatingElement.objects.create(element_type=self.element_type, score=10, comment='super')
        object_rating_1 = ObjectRating.objects.create(
            user=self.user_1, user_type=self.user_type,
            object_id=self.perm.id, content_type=self.perm_c_t,
        )
        object_rating_1.elements.add(element_1)

        element_2 = RatingElement.objects.create(element_type=self.element_type, score=9, comment='good')
        object_rating_2 = ObjectRating.objects.create(
            user=self.user_2, user_type=self.user_type,
            object_id=self.perm.id, content_type=self.perm_c_t
        )
        object_rating_2.elements.add(element_2)

        url = reverse('objectrating-list')
        response = self.user_client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.json()), 2)

    def test_get_object_rating(self):
        element = RatingElement.objects.create(element_type=self.element_type, score=10, comment='super')
        object_rating = ObjectRating.objects.create(
            user=self.user_1, user_type=self.user_type,
            object_id=self.perm.id, content_type=self.perm_c_t,
        )
        object_rating.elements.add(element)

        url = reverse('objectrating-detail', args=(object_rating.id,))
        response = self.user_client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        resp_json = response.json()
        self.assertEqual(resp_json['user_type'], self.user_type)
        self.assertEqual(resp_json['object_id'], self.perm.id)
        self.assertEqual(resp_json['content_type'], self.perm_c_t.id)
        self.assertEqual(resp_json['elements'], [element.id])

    def test_update_object_rating(self):
        element_1 = RatingElement.objects.create(element_type=self.element_type, score=10)
        element_2 = RatingElement.objects.create(element_type=self.element_type, comment='super')
        object_rating = ObjectRating.objects.create(
            user=self.user_1, user_type=self.user_type,
            object_id=self.perm.id, content_type=self.perm_c_t,
        )
        object_rating.elements.add(element_1)

        self.assertEqual(object_rating.elements.count(), 1)

        data = {'elements': [element_1.id, element_2.id]}
        url = reverse('objectrating-detail', args=(object_rating.id,))
        response = self.user_client.patch(url, data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.json()['elements'], [element_1.id, element_2.id])

    def test_delete_object_rating(self):
        element = RatingElement.objects.create(element_type=self.element_type, score=10, comment='super')
        object_rating = ObjectRating.objects.create(
            user=self.user_1, user_type=self.user_type,
            object_id=self.perm.id, content_type=self.perm_c_t,
        )
        object_rating.elements.add(element)

        url = reverse('objectrating-detail', args=(object_rating.id,))
        response = self.user_client.delete(url)
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)



import os
from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-rating',
    version='0.1.5',
    packages=find_packages(),
    include_package_data=True,
    license='BSD License',  # example license
    description='A simple Django app to handle object ratings.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://www.github.com/pulilab/django-rating',
    author='Pulilab',
    author_email='bh@pulilab.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
    ],
)



from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from .rating_values import RatingValues
from .models import ObjectRating, RatingElement


class RatingElementSerializer(serializers.ModelSerializer):

    class Meta:
        model = RatingElement
        fields = '__all__'

    @staticmethod
    def validate_element_type(value):
        if value in [item[0] for item in RatingValues.VALID_ELEMENT_TYPES]:
            return value
        raise ValidationError('Invalid element type.')

    @staticmethod
    def validate_score(value):
        if value and (value < RatingValues.MIN_SCORE or value > RatingValues.MAX_SCORE):
            raise ValidationError('Invalid score.')
        return value


class ObjectRatingSerializer(serializers.ModelSerializer):
    user = serializers.HiddenField(default=serializers.CurrentUserDefault())

    class Meta:
        model = ObjectRating
        fields = '__all__'

    @staticmethod
    def validate_user_type(value):
        if value in [item[0] for item in RatingValues.VALID_USER_TYPES]:
            return value
        raise ValidationError('Invalid user type.')
